#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


from pathlib import Path
import time
import os
import wx
import datetime as dt
import csv
import numpy as np
from nardascripting.base.signalsharkdev import *
from nardascripting.base.usrscriptbase import *
import nardascripting.base.toolbox as tb


class SavePeakTable(UsrScriptBase):
    """User script class"""

    def __init__(self, main_gui, dev=SignalSharkDev()):
        """Initialization. Please leave this code unchanged"""
        super().__init__(main_gui, dev, __file__)

        # Base settings
        self._tab_name = "nsltools"
        self._scr_title = "Save Peak Table V1.2"
        self._scr_description = "Saves current peak table as csv file"
        # self._icon_path = self.script_path.joinpath("NardaScriptLauncher_icon_close.png")
        self._list_prio = 1
        self._nsl_executed_behavior = NSL_Executed_Behaviors.MINIMIZE_NSL

    def _run_script(self, args):
        """
        Script main function
        This method is called when a user clicks on the corresponding script button.
        """
        # Connect to the device
        if not self.signalshark.connect():
            self.MessageBoxModal('Cannot connect to the device!', 'Connection error',
                                 wx.OK | wx.ICON_ERROR)
            return

        # Hold measurement acquisition
        task_state = self.signalshark.scpi.task.get_state()
        if task_state == TaskStates.RUN:
            self.signalshark.scpi.sense.hold()

        # Check if peak table exists
        # -----------------------------------------------------------------------------------------
        # Adds a new one and configures it if not.
        exists, view_index = self.signalshark.scpi.check_add_view(ViewTypes.PEAK_TABLE)
        if view_index < 1:
            self.MessageBoxModal("Adding peak table not possible.",
                                 "Peak Table", wx.OK | wx.ICON_INFORMATION)
            return

        if not exists:
            self.signalshark.scpi.display.set_peaktable_sort(PeakSort.FREQUENCY)
            self.signalshark.scpi.marker.set_spectrum_search_peak_excursion(3.0)
            self.signalshark.scpi.marker.set_spectrum_search_peak_excursion_enable(True)
            self.signalshark.scpi.marker.set_spectrum_search_threshold(-85.0)
            self.signalshark.scpi.marker.set_spectrum_search_limits_enable(True)

            self.signalshark.scpi.peaktable.get_data_update()

        # Get number of peak table rows
        # -----------------------------------------------------------------------------------------
        ptab_rows = self.signalshark.scpi.peaktable.get_data_count()
        if ptab_rows < 0:
            # e.g. show a message box:
            self.MessageBoxModal("No peak data available.", "Info",
                                 wx.OK | wx.ICON_INFORMATION)
            return

        # Ask user for CSV file name
        # -----------------------------------------------------------------------------------------
        str_meas_dir = self.config.get("meas_dir", "D:\\")
        meas_dir = Path(str_meas_dir)
        initdir = "/"

        if meas_dir.exists():
            initdir = str(meas_dir)

        # Sets a default file name consisting of a string
        # and the current date and time.
        default_file_name = dt.datetime.now().strftime("peak_table_%Y-%m-%d_%H-%M.csv")
        wildcard = "CSV file (*.csv)|*.csv"
        filename = ""
        dlg = wx.FileDialog(
            self.frm_dlghelper, message="Save peak table to",
            defaultDir=initdir,
            defaultFile=default_file_name,
            wildcard=wildcard,
            style=wx.FD_SAVE
        )
        if self.ShowDlgModalTop(dlg) == wx.ID_OK:
            filename = dlg.GetPath()
        dlg.Destroy()  # Please do not forget to destroy the created dialog if you do not need it anymore

        if filename == "":
            self.MessageBoxModal("You must enter a valid file name",
                                 "Filename", wx.OK | wx.ICON_ERROR)
            return
        
        self.config['meas_dir'] = str(Path(filename).parent)

        # Read peak table data
        # -----------------------------------------------------------------------------------------
        header = ["Fcent"]
        ptab_data = []

        # Get frequency information
        frequencies = self.signalshark.scpi.peaktable.get_data_frequency()
        ptab_data.append(frequencies)

        # Get trace data
        traces = self.signalshark.scpi.spectrum.get_trace_list()
        for trace in traces:
            header.append(str(trace))
            values = self.signalshark.scpi.peaktable.get_data_level(trace)
            ptab_data.append(values)

        # Rotate the data table content to write it to csv file rows
        # ([f1, f2], [lev1, lev2]) --> ([f1, lev1], [f2, lev2])
        ptab_data_rot = []
        for i in range(len(ptab_data[0])):
            ptab_data_rot.append([x[i] for x in ptab_data])

        # Write peak table data to csv file
        # -----------------------------------------------------------------------------------------
        with open(filename, 'w', newline='') as f:
            writer = csv.writer(f, delimiter=';')
            writer.writerow(header)
            writer.writerows(ptab_data_rot)

        # Show message:
        # -----------------------------------------------------------------------------------------
        self.MessageBoxModal("Saving peak table done!",
                             "Save peak table", wx.OK | wx.ICON_INFORMATION)
